/*\
|*| --------------------------------------------------------------------------
|*|	S3 SDAC ramdac register definitions
|*| --------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995, Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
\*/

#ifndef _S3_sdac_h
#define _S3_sdac_h

#include "IBM/vga_dac.h"

#define	DAC_SDAC_ID	0x70	/* the fourth conitinuous read	*/
#define	DAC_ID_MASK	0xF0	/* of PIXEL_MASK reg gives ID	*/
#define	DAC_REV_MASK	0x0F

#define	DAC_PLL_W_INDEX	0x4	/* PLL write index register 	*/

#define	DAC_PLL_DATA	0x5	/* PLL data (r/w) register	*/

#define	DAC_COMMAND	0x6	/* enhanced command register	*/

	#define	DAC_C_POWERDOWN	0x01
	#define	DAC_C_NOPWRSAVE	0x04
					/* res, VCLK:DCLK, PBUS	*/
	#define	DAC_C_MODE0	0x00	/*  8bpp, 1:1,  8bit	*/
	#define	DAC_C_MODE1	0x20	/* 15bpp, 2:1,  8bit	*/
	#define	DAC_C_MODE2	0x40	/* 24bpp, 3:1,  8bit	*/
	#define	DAC_C_MODE3	0x60	/* 16bpp, 2:1,  8bit	*/
	#define	DAC_C_MODE4	0x80	/* 15bpp, 2:1,  8bit	*/
	#define	DAC_C_MODE5	0xA0	/* 15bpp, 2:1,  8bit	*/
	#define	DAC_C_MODE6	0xC0	/* 16bpp, 2:1,  8bit	*/
	#define	DAC_C_MODE7	0xE0	/* 24bpp, 3:1,  8bit	*/
	#define	DAC_C_MODE8	0x10	/*  8bpp, 1:2, 16bit	*/
	#define	DAC_C_MODE9	0x30	/* 15bpp, 1:1, 16bit	*/
	#define	DAC_C_MODEA	0x50	/* 16bpp, 1:1, 16bit	*/
	#define	DAC_C_MODEB	0x70	/* 32bpp, 2:1, 16bit	*/
	#define	DAC_C_MODEC	0x90	/* 24bpp, 3:2, 16bit	*/

#define	DAC_PLL_R_INDEX	0x7	/* PLL read index register	*/

/*\
|*|	The PLL registers are accessed indirectly. First write the index
|*|	of the PLL to be programmed/read into DAC_PLL_W_INDEX/DAC_PLL_R_INDEX
|*|	and then do 2 write/read accesses to DAC_PLL_DATA. The definition of the
|*|	1st byte to be written is given by DAC_PLL0 and the 2nd by DAC_PLL1
\*/

#define	DAC_CLK0_INDEX_MASK	0x07	/* CLK 0-7 parameters, (0,1 read only */

#define	DAC_CLK1_INDEX		0x0A	/* CLK 0 parameters */

	#define	DAC_PLL0_MMASK	0x7F
	#define	DAC_PLL1_NMASK	0x1F
	#define	DAC_PLL1_DIVMASK	0x60
	#define	DAC_PLL1_DIVSHIFT	5

#define	DAC_PLL_CONTROL		0x0E

	#define	DAC_PLL_C_CLOCK_MASK	0x0F	/* Internal Clock Select mask */
	#define	DAC_PLL_C_ENB_ICS	0x20	/* ICS enable, (else CS[0:1]) */

#endif /* _S3_sdac_h */
